import time
import threading

try:
    import RPi.GPIO as GPIO
except:
    print("Error importing RPi.GPIO | Try running sudo")


class LockWithOwner:

    def __init__(self, init_owner='A'):
        self.lock = threading.RLock()
        self.is_first_time = True
        self.owner = init_owner

    def acquire_for(self, owner):
        while True:
            self.lock.acquire()
            if (self.is_first_time):
                self.is_first_time = False
                break
            if self.owner == owner:
                break
            self.lock.release()
            # time.sleep(0.001)

    def release_to(self, new_owner):
        self.owner = new_owner
        self.lock.release()


class UltraSound(threading.Thread):

    def __init__(self, l: LockWithOwner) -> None:
        threading.Thread.__init__(self)
        self.lock_global: LockWithOwner = l
        self.lock = LockWithOwner()
        self.lock.owner = 'A'

        GPIO.setmode(GPIO.BCM)

        GPIO_TRIGGER: int = 18
        GPIO_ECHO: int = 24
        GPIO_TRIGGER2: int = 19
        GPIO_ECHO2: int = 17

        GPIO.setup(GPIO_TRIGGER, GPIO.OUT)
        GPIO.setup(GPIO_ECHO, GPIO.IN)
        GPIO.setup(GPIO_TRIGGER2, GPIO.OUT)
        GPIO.setup(GPIO_ECHO2, GPIO.IN)

        self.threadA: threading.Thread = threading.Thread(target=self.distance, args=(self,
                                                                                      GPIO_TRIGGER, GPIO_ECHO, 'A'))
        self.threadB: threading.Thread = threading.Thread(target=self.distance, args=(self,
                                                                                      GPIO_TRIGGER2, GPIO_ECHO2, 'B'))

    def run(self):
        self.threadA.start()
        self.threadB.start()
        self.threadA.join()
        self.threadB.join()
        GPIO.cleanup()

    def distance(self, trigger, echo, entry):
        while True:
            if (self.lock_global.owner == "SENSOR"):
                self.lock.acquire_for(entry)
                # Set trigger to HIGH
                GPIO.output(trigger, True)
                time.sleep(0.001)
                GPIO.output(trigger, False)
                start_time = time.time()
                stop_time = time.time()
                while GPIO.input(echo) == 0:
                    start_time = time.time()
                while GPIO.input(echo) == 1:
                    stop_time = time.time()
                time_elapsed: int = stop_time - start_time
                distance = (time_elapsed * 34300) / 2
                if distance < 15:
                    print(entry)
                time.sleep(0.1)
                if (entry == 'B'):
                    self.lock.release_to('A')
                else:
                    self.lock.release_to('B')
                self.lock_global.release_to("CAM")
