from db import EventObject, WebDB
import socket
import threading
import _thread
import time


class WebServer(threading.Thread):
    clients: list["WebClient"] = []
    database: WebDB = WebDB()

    def __init__(self, host_address="127.0.0.1", port_number: int = 65432) -> None:
        """Start the server"""
        threading.Thread.__init__(self)
        self.host_address = host_address
        self.port_number = port_number
        self.ServerSocket: socket.socket
        # test_if_alive: TestIfAlive = TestIfAlive()
        # test_if_alive.start()

    def run(self):
        self.ServerSocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.ServerSocket.bind((self.host_address, self.port_number))
        self.ServerSocket.listen(5)

        while True:
            client, address = self.ServerSocket.accept()
            print('Connected to: ' + address[0] + ':' + str(address[1]))
            web_client = WebClient(
                sock=client, address=f"{address[0]}:{address[1]}")
            WebServer.clients.append(web_client)
            web_client.start()

    @staticmethod
    def remove_client(client: "WebClient"):
        WebServer.clients.remove(client)

    @staticmethod
    def get_database_entry(id: int) -> EventObject | None:
        return WebServer.database.get(id=id)

    @staticmethod
    def remove_database_entry(id: int) -> None:
        return WebServer.database.remove(id=id)

    @staticmethod
    def update_database_entry(document: dict) -> None:
        return WebServer.database.update(event=EventObject.from_dict(document=document))

    @staticmethod
    def add_database_entry(document: dict) -> EventObject:
        return WebServer.database.insert(event=EventObject.from_dict(document=document))


class WebClient(threading.Thread):
    def __init__(self, sock, address) -> None:
        threading.Thread.__init__(self)
        self.address = address
        self.sock: socket.socket = sock

    def run(self):
        self.sock.sendall(str.encode("Successfully connected!"))
        while True:
            message = self.sock.recv(2048)
            try:
                self.handle_request(message=message)
            except:
                self.sock.close()
                WebServer.remove_client(client=self)
                break

    def send_message(self, message: str):
        self.sock.sendall(str.encode(message))

    def handle_request(self, message: bytes):
        data = message.decode("utf-8")
        if (data == "[PONG]"):
            self.send_message(message="[PING]")
        elif (data.startswith("GET_ID ")):
            print("Here")
            entry = WebServer.get_database_entry(
                id=int(data.replace("GET_ID ", '')))
            if (entry):
                self.send_message(message=entry.to_dict())
            else:
                self.send_message("NULL")
        elif (data.startswith("REMOVE_ID ")):
            WebServer.remove_database_entry(
                id=int(data.replace("REMOVE_ID ", '')))
        elif (data.startswith("UPDATE ")):
            WebServer.update_database_entry(
                document=data.replace("UPDATE ", ''))
        elif (data.startswith("ADD ")):
            self.send_message(message=WebServer.add_database_entry(
                document=data.replace("ADD ", '')).to_dict())
        else:
            pass
            # print(f"ERROR | unknown command: {data}")


class TestIfAlive(threading.Thread):
    def __init__(self):
        threading.Thread.__init__(self)

    def run(self):
        while True:
            for client in WebServer.clients:
                _thread.start_new_thread(self.ping_client, (client, None))
            time.sleep(50)

    def ping_client(client: WebClient, __, ___) -> None:
        client.send_message(message="[PING]")
        return time.sleep(20)
