import { Component, Input, OnDestroy } from '@angular/core';
import { StreamingServiceService } from 'src/app/streaming-service.service';
import { Subscriber, Subscription } from "rxjs";

@Component({
  selector: 'app-capture-highlight',
  templateUrl: './capture-highlight.component.html',
  styleUrls: ['./capture-highlight.component.css']
})
export class CaptureHighlightComponent implements OnDestroy {
  stream_is_active: boolean = false;
  stream_source: string = "http://0.0.0.0:8000/index.html";
  streaming_subscription: Subscription | undefined;
  intervalSourceCheck: any;
  show_change_source: boolean = false;

  constructor(private streaming_service: StreamingServiceService) {
    this.streaming_subscription = this.streaming_service.is_stream_active.subscribe((val: Map<string, any>) => {
      this.stream_is_active = val.get('is_active') == "YES" ? true : false;
      this.stream_source = val.get('source') ?? "";
    });
  }

  showChangeSource() {
    this.show_change_source = !this.show_change_source;
  }

  changeSource(url: string | undefined) {
    this.stream_source = url ?? this.stream_source;
    this.streaming_service.changeSource(url);
    this.show_change_source = false;
  }


  ngOnDestroy() {
    if (this.streaming_subscription) {
      this.streaming_subscription.unsubscribe()
    }
  }

}
