import { Injectable } from '@angular/core';
import { Observable, Observer, Subscriber } from "rxjs";


@Injectable({
  providedIn: 'root'
})
export class StreamingServiceService {
  stream_source: string = "http://192.168.137.41:8000/index.html";
  is_stream_active: Observable<Map<string, any>> = new Observable()

  constructor() {
    this.is_stream_active = new Observable(this.multicastSequenceSubscriber());
  }

  multicastSequenceSubscriber() {
    const observers: Observer<Map<string, any>>[] = [];
    // Return the subscriber function (runs when subscribe()
    // function is invoked)
    return (observer: Observer<Map<string, any>>) => {
      observers.push(observer);
      // When this is the first subscription, start the sequence
      if (observers.length === 1) {
        const multicastObserver: Observer<Map<string, any>> = {
          next(val) {
            // Iterate through observers and notify all subscriptions
            observers.forEach(obs => obs.next(val));
          },
          error() { /* Handle the error... */ },
          complete() {
            // Notify all complete callbacks
            observers.slice(0).forEach(obs => obs.complete());
          }
        };
        this.doSequence(multicastObserver);
      }

      return {
        unsubscribe() {
          // Remove from the observers array so it's no longer notified
          observers.splice(observers.indexOf(observer), 1);
          // If there's no more listeners, do cleanup
        }
      };
    }
  }

  doSequence(sequenceObserver: Observer<Map<string, any>>) {
    setInterval(() => {
      let map = new Map<string, any>();
      fetch(this.stream_source, { mode: "no-cors" })
        .then(res => {
          map.set('source', this.stream_source)
          map.set('is_active', "YES")
          sequenceObserver.next(map)
        })
        .catch(err => {
          map.set('source', this.stream_source)
          map.set('is_active', "NO")
          sequenceObserver.next(map)
        })
    }
      , 10_000);
  }

  changeSource(url: string | undefined) {
    this.stream_source = url ?? this.stream_source;
  }

}
